/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[uuid(d9539ecb-6665-452c-bae7-4e42f25d23aa)]
interface nsIServiceWorkerManager : nsISupports
{
  // Returns a Promise
  nsISupports register(in nsIDOMWindow aWindow, in DOMString aScope, in DOMString aScriptURI);

  // Returns a Promise
  nsISupports unregister(in nsIDOMWindow aWindow, in DOMString aScope);

};

%{ C++
#define SERVICEWORKERMANAGER_CONTRACTID "@mozilla.org/serviceworkers/manager;1"
%}
