/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * The nsIHTTPHeaderListener interface allows plugin authors to
 * access HTTP Response headers after issuing an
 * nsIPluginHost::{GetURL,PostURL}() call. <P>
 */

[scriptable, uuid(ea51e0b8-871c-4b85-92da-6f400394c5ec)]
interface nsIHTTPHeaderListener : nsISupports
{
  /**
   * Called for each HTTP Response header.
   * NOTE: You must copy the values of the params.  
   */
  void newResponseHeader(in string headerName, in string headerValue);

  /**
   * Called once for the HTTP Response status line.
   * Value does NOT include a terminating newline.
   * NOTE: You must copy this value.
   */
  void statusLine(in string line);
};
