/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIContentPermissionRequest;

/**
 * Generic permission service for access to WebAPIs, hardware, capabilities.
 */
[scriptable, uuid(e5f953b3-a6ca-444e-a88d-cdc81383741c)]
interface nsIPermissionPromptService : nsISupports 
{
  /**
   * Checks if the capability requires a permission, fires the corresponding cancel() 
   * or allow() method in aRequest after consulting PermissionSettings, etc.
   */
  void getPermission(in nsIContentPermissionRequest aRequest);
};
