/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIInputStream;
interface nsIUnicharInputStream;

/**
 * Factory to create objects that implement nsIUnicharInputStream,
 * converting from a unicode string or a UTF-8 stream.
 */
[scriptable, uuid(8238cd2e-e8e3-43e8-b556-56e21389c766)]
interface nsISimpleUnicharStreamFactory : nsISupports
{
  /**
   * Create a unicode input stream from a unicode string.
   */
  nsIUnicharInputStream createInstanceFromString(in AString aString);

  /**
   * Create a unicode stream from an input stream in UTF8.
   */
  nsIUnicharInputStream createInstanceFromUTF8Stream(in nsIInputStream aStream);
};
